@@ -52,7 +52,7 @@ void IVariableController::stateChanged(IDebugSession::DebuggerState state)
         //variables are now outdated, update them
         m_activeThread = -1;
         m_activeFrame = -1;
-    } else if (state == IDebugSession::StoppedState) {
+    } else if (state == IDebugSession::EndedState || state == IDebugSession::NotStartedState) {
         // Remove all locals.
         foreach (Locals *l, variableCollection()->allLocals()) {
             l->deleteChildren();
@@ -86,7 +86,9 @@ void IVariableController::handleEvent(IDebugSession::event_t event)
         kDebug() << m_autoUpdate;
         if (!(m_autoUpdate & UpdateLocals)) {
             foreach (Locals *l, variableCollection()->allLocals()) {
-                l->setHasMore(true);
+                if (!l->isExpanded() && !l->childCount()) {
+                    l->setHasMore(true);
+                }
             }
         }
         if (m_autoUpdate != UpdateNone) {
