@@ -18,6 +18,7 @@
 */
 
 #include "idocumentation.h"
+#include <qtextbrowser.h>
 
 namespace KDevelop
 {
@@ -25,4 +26,12 @@ namespace KDevelop
 IDocumentation::~IDocumentation()
 {}
 
+QWidget* IDocumentation::documentationWidget(QWidget* parent)
+{
+    QTextBrowser* widget=new QTextBrowser(parent);
+    widget->setReadOnly(true);
+    widget->setText(description());
+    return widget;
+}
+
 }
