@@ -23,6 +23,7 @@
 #include <language/duchain/indexedstring.h>
 #include <ksharedptr.h>
 #include <KDE/KUrl>
+#include "coderepresentation.h"
 
 namespace KDevelop {
     
@@ -49,13 +50,16 @@ public:
 
 typedef KSharedPtr<DocumentChange> DocumentChangePointer;
 
+/**
+ * Object representing an arbitrary set of changes to an arbitrary set of files that can be applied atomically.
+ */
 class KDEVPLATFORMLANGUAGE_EXPORT DocumentChangeSet {
   public:
     
-      DocumentChangeSet();
-      DocumentChangeSet(const DocumentChangeSet & rhs);
-      ~DocumentChangeSet();
-      DocumentChangeSet& operator=(const DocumentChangeSet& rhs);
+    DocumentChangeSet();
+    ~DocumentChangeSet();
+    DocumentChangeSet(const DocumentChangeSet & rhs);
+    DocumentChangeSet& operator=(const DocumentChangeSet& rhs);
     
     //Returns true on success
     struct ChangeResult {
@@ -76,55 +80,50 @@ class KDEVPLATFORMLANGUAGE_EXPORT DocumentChangeSet {
     
     ///If the change has multiple lines, a problem will be returned. these don't work at he moment.
     ChangeResult addChange(const DocumentChange& change);
-    ///Get rid of all the current changes
-    void clear();
-    
-    ///Merge two document change sets
-    DocumentChangeSet & operator<<(DocumentChangeSet &);
     
     enum ReplacementPolicy {
         IgnoreFailedChange,///If this is given, all changes that could not be applied are simply ignored
         WarnOnFailedChange,///If this is given to applyAllChanges, a warning is given when a change could not be applied,
                             ///but following changes are applied, and success is returned.
-        StopOnFailedChange ///If this is given to applyAllChanges, then all replacements are reverted and an error returned on problems
+        StopOnFailedChange ///If this is given to applyAllChanges, then all replacements are reverted and an error returned on problems (default)
     };
+    
     ///@param policy What should be done when a change could not be applied?
     void setReplacementPolicy(ReplacementPolicy policy);
     
     enum FormatPolicy {
         NoAutoFormat, ///If this option is given, no automatic formatting is applied
-        AutoFormatChanges      ///If this option is given, all changes are automatically reformatted using the formatter plugin for the mime type
+        AutoFormatChanges      ///If this option is given, all changes are automatically reformatted using the formatter plugin for the mime type (default)
     };
+    
     ///@param policy How the changed text should be formatted
     void setFormatPolicy(FormatPolicy policy);
     
     enum DUChainUpdateHandling {
         NoUpdate,       ///No updates will be scheduled
-        SimpleUpdate ///The changed documents will be added to the background parser, plus all documents that are currently open and recursively import those documetns
+        SimpleUpdate ///The changed documents will be added to the background parser, plus all documents that are currently open and recursively import those documents (default)
         //FullUpdate       ///All documents in all open projects that recursively import any of the changed documents will be updated
     };
+    
     ///@param policy Whether a duchain update should be triggered for all affected documents
     void setUpdateHandling(DUChainUpdateHandling policy);
     
-    ///Apply all the registered changes for @p file, and keep them as temporary in memory
-    ///@note For multiple passes if the original file is given, then the previous temporary will be merged
-    ChangeResult applyToTemp(IndexedString file);
+    enum ActivationPolicy {
+        Activate,           ///The affected files will be activated
+        DoNotActivate  ///The affected files will not be activated (default)
+    };
     
-    ///Apply all changes for all files, and keep them as temporary in memory
-    ChangeResult applyAllToTemp();
+    ///@param policy Whether the affected documents should be activated when the change is applied
+    void setActivationPolicy(ActivationPolicy policy);
     
-    /// @return The name for the latest temporary version of @p file if it exists, returns @p file otherwise
-    IndexedString tempNameForFile(IndexedString file) const;
+    ///Applies all changes to temporary code representations, and returns a map from each file-name to
+    ///the respective inserted artificial code-representation.
+    QMap<IndexedString, InsertArtificialCodeRepresentationPointer> temporaryCodeRepresentations() const;
     
-    /// @return The mapping between all original file names and their temporaries
-    /// @note If a file has not been applied to temp, it won't appear in the mapping
-    QMap<IndexedString, IndexedString> tempNamesForAll() const;
-    
-    /// Apply all the changes registered in this changeset
+    /// Apply all the changes registered in this changeset to the actual files
     ChangeResult applyAllChanges();
     
   private:
-        
     DocumentChangeSetPrivate * d;
 };
 }
