@@ -61,7 +61,7 @@ typedef KSharedPtr<AbstractNavigationContext> NavigationContextPointer;
 class KDEVPLATFORMLANGUAGE_EXPORT AbstractNavigationContext : public KShared
 {
   public:
-    AbstractNavigationContext( KDevelop::TopDUContextPointer topContext, AbstractNavigationContext* previousContext = 0 );
+    AbstractNavigationContext( KDevelop::TopDUContextPointer topContext = KDevelop::TopDUContextPointer(), AbstractNavigationContext* previousContext = 0 );
 
     virtual ~AbstractNavigationContext() {
     }
@@ -85,13 +85,25 @@ class KDEVPLATFORMLANGUAGE_EXPORT AbstractNavigationContext : public KShared
     ///The widget may have a signal "navigateDeclaration(KDevelop::IndexedDeclaration)".
     ///If that signal is emitted, the new declaration is navigated in the navigation-wdiget.
     virtual QWidget* widget() const;
+    
+    ///Whether the widget returned by widget() should take the maximum possible spsace.
+    ///The default implementation returns true.
+    virtual bool isWidgetMaximized() const;
 
     ///Returns whether this context's string has already been computed, and is up to date.
     ///After clear() was called, this returns false again.
     bool alreadyComputed() const;
     
+    void setTopContext(TopDUContextPointer context);
+
+    TopDUContextPointer topContext() const;
+
+    NavigationContextPointer executeLink(QString link);
+    
   protected:
     
+    virtual void setPreviousContext(AbstractNavigationContext* previous);
+    
     struct TextHandler {
       TextHandler(AbstractNavigationContext* c) : context(c) {
       }
@@ -101,7 +113,8 @@ class KDEVPLATFORMLANGUAGE_EXPORT AbstractNavigationContext : public KShared
       AbstractNavigationContext* context;
     };
     
-    
+    ///Override this to execute own key-actions using NavigationAction
+    virtual NavigationContextPointer executeKeyAction(QString key);
 
     ///Adds given the text to currentHtml()
     void addHtml(QString html);
@@ -115,15 +128,12 @@ class KDEVPLATFORMLANGUAGE_EXPORT AbstractNavigationContext : public KShared
     //Clears the computed html and links
     void clear();
     
-    ///Override this to execute own key-actions using NavigationAction
-    virtual NavigationContextPointer executeKeyAction(QString key);
-    
     NavigationContextPointer execute(NavigationAction& action);
 
     void addExternalHtml( const QString& text );
 
     ///Creates and registers a link to the given declaration, labeled by the given name
-    void makeLink( const QString& name, DeclarationPointer declaration, NavigationAction::Type actionType );
+    virtual void makeLink( const QString& name, DeclarationPointer declaration, NavigationAction::Type actionType );
 
     ///Creates a link that executes the given action
     void makeLink( const QString& name, QString targetId, const NavigationAction& action);
@@ -135,6 +145,8 @@ class KDEVPLATFORMLANGUAGE_EXPORT AbstractNavigationContext : public KShared
     NavigationContextPointer registerChild( AbstractNavigationContext* context );
     QList<NavigationContextPointer> m_children; //Useed to keep alive all children until this is deleted
 
+    bool m_shorten;
+
     int m_currentLine;
 
     //A counter used while building the html-code to count the used links.
@@ -148,8 +160,9 @@ class KDEVPLATFORMLANGUAGE_EXPORT AbstractNavigationContext : public KShared
     QString m_prefix, m_suffix;
     KDevelop::TopDUContextPointer m_topContext;
 
-    static QString declarationKind(DeclarationPointer decl);
+    virtual QString declarationKind(DeclarationPointer decl);
 
+    static const Colorizer typeHighlight;
     static const Colorizer errorHighlight;
     static const Colorizer labelHighlight;
     static const Colorizer codeHighlight;
