@@ -31,7 +31,7 @@ namespace KDevelop
 
 REGISTER_TYPE(PointerType);
 
-PointerType::PointerType(const PointerType& rhs) : AbstractType(copyData<PointerTypeData>(*rhs.d_func())) {
+PointerType::PointerType(const PointerType& rhs) : AbstractType(copyData<PointerType>(*rhs.d_func())) {
 }
 
 PointerType::PointerType(PointerTypeData& data) : AbstractType(data) {
@@ -57,21 +57,20 @@ bool PointerType::equals(const AbstractType* _rhs) const
 }
 
 PointerType::PointerType()
-  : AbstractType(createData<PointerTypeData>())
+  : AbstractType(createData<PointerType>())
 {
-  d_func_dynamic()->setTypeClassId<PointerType>();
 }
 
 void PointerType::accept0 (TypeVisitor *v) const
 {
   if (v->visit (this))
-    acceptType (d_func()->m_baseType.type(), v);
+    acceptType (d_func()->m_baseType.abstractType(), v);
 
   v->endVisit (this);
 }
 
 void PointerType::exchangeTypes( TypeExchanger* exchanger ) {
-  d_func_dynamic()->m_baseType = exchanger->exchange( d_func()->m_baseType.type() )->indexed();
+  d_func_dynamic()->m_baseType = exchanger->exchange( d_func()->m_baseType.abstractType() )->indexed();
 }
 
 PointerType::~PointerType()
@@ -80,7 +79,7 @@ PointerType::~PointerType()
 
 AbstractType::Ptr PointerType::baseType () const
 {
-  return d_func()->m_baseType.type();
+  return d_func()->m_baseType.abstractType();
 }
 
 void PointerType::setBaseType(AbstractType::Ptr type)
@@ -90,7 +89,8 @@ void PointerType::setBaseType(AbstractType::Ptr type)
 
 QString PointerType::toString() const
 {
-  return (baseType() ? QString("%1*").arg(baseType()->toString()) : QString("<notype>*")) + AbstractType::toString(true);
+  QString baseString = (baseType() ? baseType()->toString() : "<notype>");
+  return QString("%1*").arg(baseString) + AbstractType::toString(true);
 }
 
 AbstractType::WhichType PointerType::whichType() const
