@@ -25,6 +25,7 @@
 #include <interfaces/iruncontroller.h>
 #include <language/duchain/indexedstring.h>
 #include <project/importprojectjob.h>
+#include <project/helper.h>
 #include <interfaces/iuicontroller.h>
 
 #include <kparts/mainwindow.h>
@@ -50,6 +51,8 @@
 
 #include "genericmanagerlistjob.h"
 
+#define ifDebug(x)
+
 K_PLUGIN_FACTORY(GenericSupportFactory, registerPlugin<GenericProjectManager>(); )
 K_EXPORT_PLUGIN(GenericSupportFactory(KAboutData("kdevgenericmanager","kdevgenericprojectmanager",ki18n("Generic Project Manager"), "0.1", ki18n("A plugin to support basic project management on a filesystem level"), KAboutData::License_GPL)))
 
@@ -65,12 +68,32 @@ GenericProjectManager::IncludeRules getIncludeRules(KDevelop::IProject* project)
     return qMakePair(includes, excludes);
 }
 
+/**
+ * Returns the parent folder item for a given item or the project root item if there is no parent.
+ */
+KDevelop::ProjectFolderItem* getParentFolder(KDevelop::ProjectBaseItem* item)
+{
+    if ( item->parent() ) {
+        return static_cast<KDevelop::ProjectFolderItem*>(item->parent());
+    } else {
+        return item->project()->projectItem();
+    }
+}
+
 GenericProjectManager::GenericProjectManager( QObject *parent, const QVariantList & args )
         : KDevelop::IPlugin( GenericSupportFactory::componentData(), parent ), KDevelop::IGenericProjectManager()
 {
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IProjectFileManager )
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IGenericProjectManager )
     Q_UNUSED( args )
+
+    connect(core()->projectController(), SIGNAL(projectClosing(KDevelop::IProject*)),
+            this, SLOT(projectClosing(KDevelop::IProject*)));
+}
+
+void GenericProjectManager::projectClosing(KDevelop::IProject* project)
+{
+    delete m_watchers.take(project);
 }
 
 GenericProjectManager::~GenericProjectManager()
@@ -182,8 +205,8 @@ void GenericProjectManager::addJobItems(KDevelop::ProjectFolderItem* baseItem, c
         }
     }
 
-    kDebug() << "valid folders:" << folders;
-    kDebug() << "valid files:" << files;
+    ifDebug(kDebug() << "valid folders:" << folders;)
+    ifDebug(kDebug() << "valid files:" << files;)
 
     // remove obsolete rows
     for ( int j = 0; j < baseItem->rowCount(); ++j ) {
@@ -193,7 +216,7 @@ void GenericProjectManager::addJobItems(KDevelop::ProjectFolderItem* baseItem, c
             int index = folders.indexOf( f->url() );
             if ( index == -1 ) {
                 // folder got removed or is now invalid
-                kDebug() << "removing folder:" << f->url();
+                ifDebug(kDebug() << "removing folder:" << f->url();)
                 baseItem->removeRow( j );
                 --j;
             } else {
@@ -210,8 +233,7 @@ void GenericProjectManager::addJobItems(KDevelop::ProjectFolderItem* baseItem, c
             int index = files.indexOf( f->url() );
             if ( index == -1 ) {
                 // file got removed or is now invalid
-                kDebug() << "removing file:" << f->url();
-                baseItem->project()->removeFromFileSet( KDevelop::IndexedString( f->url() ) );
+                ifDebug(kDebug() << "removing file:" << f->url();)
                 baseItem->removeRow( j );
                 --j;
             } else {
@@ -224,7 +246,6 @@ void GenericProjectManager::addJobItems(KDevelop::ProjectFolderItem* baseItem, c
     // add new rows
     foreach ( const KUrl& url, files ) {
         new KDevelop::ProjectFileItem( baseItem->project(), url, baseItem );
-        baseItem->project()->addToFileSet( KDevelop::IndexedString( url ) );
     }
     foreach ( const KUrl& url, folders ) {
         emit appendSubDir( new KDevelop::ProjectFolderItem( baseItem->project(), url, baseItem ) );
@@ -266,6 +287,7 @@ bool GenericProjectManager::reload( KDevelop::ProjectFolderItem* item )
 
 void GenericProjectManager::created(const QString &path)
 {
+    kDebug() << "created:" << path;
     QFileInfo info(path);
 
     KUrl url = KUrl(path);
@@ -275,8 +297,9 @@ void GenericProjectManager::created(const QString &path)
         if ( !isValid(url, info.isDir(), p, getIncludeRules(p)) ) {
             continue;
         }
-        if ( !p->foldersForUrl(url).isEmpty() ) {
+        if ( !p->foldersForUrl(url).isEmpty() || !p->filesForUrl(url).isEmpty() ) {
             // exists already in this project, happens e.g. when we restart the dirwatcher
+            // for files it also gets triggered for kate's backup files
             continue;
         }
         foreach ( KDevelop::ProjectFolderItem* parentItem, p->foldersForUrl(parent) ) {
@@ -284,7 +307,6 @@ void GenericProjectManager::created(const QString &path)
                 eventuallyReadFolder(new KDevelop::ProjectFolderItem( p, url, parentItem ));
             } else {
                 new KDevelop::ProjectFileItem( p, url, parentItem );
-                p->addToFileSet( KDevelop::IndexedString( url ) );
             }
         }
     }
@@ -296,6 +318,7 @@ void GenericProjectManager::deleted(const QString &path)
         // stopDirScan...
         return;
     }
+    kDebug() << "deleted:" << path;
 
     KUrl url = KUrl(path);
     foreach ( KDevelop::IProject* p, m_watchers.keys() ) {
@@ -316,28 +339,30 @@ void GenericProjectManager::deleted(const QString &path)
 }
 
 KDevelop::ProjectFolderItem* GenericProjectManager::addFolder( const KUrl& url,
-        KDevelop::ProjectFolderItem * folder )
+        KDevelop::ProjectFolderItem * parent )
 {
-    // dirwatcher cares about local folders, yet remote ones have to be added by hand
-    KDevelop::ProjectFolderItem* newFolder = 0;
-    if ( !url.isLocalFile() ) {
-        kDebug() << "adding remote folder" << url << "to" << folder->url();
-        newFolder = new KDevelop::ProjectFolderItem( folder->project(), url, folder );
+    kDebug() << "adding folder" << url << "to" << parent->url();
+    KDevelop::ProjectFolderItem* created = 0;
+    stopWatcher(parent);
+    if ( KDevelop::createFolder(url) ) {
+        created = new KDevelop::ProjectFolderItem( parent->project(), url, parent );
     }
-    return newFolder;
+    continueWatcher(parent);
+    return created;
 }
 
 
 KDevelop::ProjectFileItem* GenericProjectManager::addFile( const KUrl& url,
-        KDevelop::ProjectFolderItem * folder )
+        KDevelop::ProjectFolderItem * parent )
 {
-    // dirwatcher cares about local files, yet remote ones have to be added by hand
-    KDevelop::ProjectFileItem* file = 0;
-    if ( !url.isLocalFile() ) {
-        kDebug() << "adding remote file" << url << "to" << folder->url();
-        file = new KDevelop::ProjectFileItem( folder->project(), url, folder );
+    kDebug() << "adding file" << url << "to" << parent->url();
+    KDevelop::ProjectFileItem* created = 0;
+    stopWatcher(parent);
+    if ( KDevelop::createFile(url) ) {
+        created = new KDevelop::ProjectFileItem( parent->project(), url, parent );
     }
-    return file;
+    continueWatcher(parent);
+    return created;
 }
 
 bool GenericProjectManager::renameFolder( KDevelop::ProjectFolderItem * folder, const KUrl& url )
@@ -356,8 +381,6 @@ bool GenericProjectManager::renameFile( KDevelop::ProjectFileItem * file, const
     kDebug() << "trying to rename a file:" << file->url() << url;
 
     if ( rename(file, file->url(), url) ) {
-        file->project()->removeFromFileSet( KDevelop::IndexedString( file->url() ) );
-        file->project()->addToFileSet( KDevelop::IndexedString( url ) );
         file->setUrl(url);
         return true;
     } else {
@@ -396,22 +419,28 @@ bool GenericProjectManager::rename(KDevelop::ProjectBaseItem* item, const KUrl&
 
 bool GenericProjectManager::removeFolder( KDevelop::ProjectFolderItem * folder )
 {
-    // dirwatcher cares about local folders, yet remote ones have to be removed by hand
-    if ( !folder->url().isLocalFile() ) {
-        kDebug() << "removing folder" << folder->url();
+    kDebug() << "removing folder" << folder->url();
+    KDevelop::ProjectFolderItem* parent = getParentFolder(folder);
+    stopWatcher(parent);
+    const bool success = KDevelop::removeUrl(folder->url(), true);
+    if ( success ) {
         folder->parent()->removeRow( folder->row() );
     }
-    return true;
+    continueWatcher(parent);
+    return success;
 }
 
 bool GenericProjectManager::removeFile( KDevelop::ProjectFileItem * file )
 {
-    // dirwatcher cares about local files, yet remote ones have to be removed by hand
-    if ( !file->url().isLocalFile() ) {
-        kDebug() << "removing file" << file->url();
+    kDebug() << "removing file" << file->url();
+    KDevelop::ProjectFolderItem* parent = getParentFolder(file);
+    stopWatcher(parent);
+    const bool success = KDevelop::removeUrl(file->url(), false);
+    if ( success ) {
         file->parent()->removeRow( file->row() );
     }
-    return true;
+    continueWatcher(parent);
+    return success;
 }
 
 void GenericProjectManager::stopWatcher(KDevelop::ProjectFolderItem* folder)
