@@ -40,7 +40,6 @@ Copyright 2006-2009 David Nolden <david.nolden.kdevelop@art-master.de>
 #include <kmessagebox.h>
 #include <QMetaType>
 #include <QVariant>
-#include "libdiff2/diffsettings.h"
 #include <ktexteditor/cursor.h>
 #include <ktexteditor/document.h>
 #include <ktexteditor/view.h>
@@ -70,18 +69,25 @@ std::string*/
 #include <sublime/mainwindow.h>
 #include <sublime/area.h>
 #include <interfaces/iprojectcontroller.h>
+#include "diffsettings.h"
 
 using namespace KDevelop;
 
+namespace {
+  // Maximum number of files to open directly within a tab when the review is started
+  const int maximumFilesToOpenDirectly = 15;
+}
+
 Q_DECLARE_METATYPE( const Diff2::DiffModel* )
 
 PatchReviewToolView::PatchReviewToolView( QWidget* parent, PatchReviewPlugin* plugin ) 
 : QWidget( parent ), m_reversed( false ), m_plugin( plugin ) 
 {
-    showEditDialog();
     connect( plugin, SIGNAL(patchChanged()), SLOT(patchChanged()) );
+    connect( ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), this, SLOT(documentActivated(KDevelop::IDocument*)) );
+    
+    showEditDialog();
     patchChanged();
-    connect( ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), this, SLOT(documentActivated(KDevelop::IDocument*)));
 }
 
 void PatchReviewToolView::patchChanged()
@@ -177,6 +183,7 @@ void PatchReviewToolView::fillEditFromPatch() {
 
 void PatchReviewToolView::patchSelectionChanged(int selection)
 {
+  m_editPatch.filesList->clear();
     if(selection >= 0 && selection < m_plugin->knownPatches().size()) {
       m_plugin->setPatch(m_plugin->knownPatches()[selection]);
     }
@@ -260,7 +267,7 @@ void PatchReviewPlugin::seekHunk( bool forwards, const KUrl& fileName ) {
         if ( !m_modelList.get() )
             throw "no model";
 
-        for (uint a = 0; a < m_modelList->modelCount(); ++a) {
+        for (int a = 0; a < m_modelList->modelCount(); ++a) {
 
             const Diff2::DiffModel* model = m_modelList->modelAt(a);
             if ( !model || !model->differences() )
@@ -330,7 +337,7 @@ void PatchReviewPlugin::addHighlighting(const KUrl& highlightFile, IDocument* do
         if ( !modelList() )
             throw "no model";
 
-        for (uint a = 0; a < modelList()->modelCount(); ++a) {
+        for (int a = 0; a < modelList()->modelCount(); ++a) {
             const Diff2::DiffModel* model = modelList()->modelAt(a);
             if ( !model )
                 continue;
@@ -371,7 +378,7 @@ void PatchReviewPlugin::highlightPatch() {
         if ( !modelList() )
             throw "no model";
 
-        for (uint a = 0; a < modelList()->modelCount(); ++a) {
+        for (int a = 0; a < modelList()->modelCount(); ++a) {
             const Diff2::DiffModel* model = modelList()->modelAt(a);
             if ( !model )
                 continue;
@@ -395,10 +402,11 @@ void PatchReviewPlugin::highlightPatch() {
 void PatchReviewToolView::finishReview()
 {
     QList<KUrl> selectedUrls;
-    for(uint a = 0; a< m_editPatch.filesList->topLevelItemCount(); ++a) {
+    for(int a = 0; a< m_editPatch.filesList->topLevelItemCount(); ++a) {
       QTreeWidgetItem* item = m_editPatch.filesList->topLevelItem(a);
       if(item && item->checkState(0) == Qt::Checked) {
         QVariant v = item->data(0, Qt::UserRole);
+        
         if( v.canConvert<KUrl>() ) {
           selectedUrls << v.value<KUrl>();
         }else if ( v.canConvert<const Diff2::DiffModel*>() ) {
@@ -475,35 +483,34 @@ void PatchReviewToolView::kompareModelChanged()
     
     const Diff2::DiffModelList* models = m_plugin->modelList()->models();
     if ( !models )
-        throw "no diff-models";
+        return;
     Diff2::DiffModelList::const_iterator it = models->constBegin();
     QSet<KUrl> haveUrls;
-    while ( it != models->constEnd() ) {
+    for(; it != models->constEnd(); ++it) {
         Diff2::DifferenceList * diffs = ( *it ) ->differences();
         int cnt = 0;
         if ( diffs )
             cnt = diffs->count();
 
         KUrl file = urlForFileModel(*it);
+        haveUrls.insert(file);
 
+        if(!QFileInfo(file.toLocalFile()).isReadable())
+          continue;
+          
         QTreeWidgetItem* item = new QTreeWidgetItem(m_editPatch.filesList);
         
-        haveUrls.insert(file);
-        
         m_editPatch.filesList->insertTopLevelItem(0, item);
-        QString text = QString( "%1 (%2 hunks" ).arg( ICore::self()->projectController()->prettyFileName(file, KDevelop::IProjectController::FormatPlain) ).arg( cnt );
+        QString text = i18n( "%1 (%2 hunks", ICore::self()->projectController()->prettyFileName(file, KDevelop::IProjectController::FormatPlain), cnt );
         
         if(additionalUrls.contains(file) && !additionalUrls[file].isEmpty())
-          text += ", " + additionalUrls[file];
+          text += i18n(", %1", additionalUrls[file]);
         
-        text += ")";
+        text += i18n(")");
         
         item->setData( 0, Qt::DisplayRole, text );
-        QVariant v;
-        v.setValue<const Diff2::DiffModel*>( *it );
-        item->setData( 0, Qt::UserRole, v );
+        item->setData( 0, Qt::UserRole, qVariantFromValue<const Diff2::DiffModel*>(*it));
         item->setCheckState( 0, Qt::Checked );
-        ++it;
     }
     
     ///First add files that a project was found for, then the other ones
@@ -544,7 +551,7 @@ void PatchReviewToolView::documentActivated(IDocument* doc)
     QModelIndexList i = m_editPatch.filesList->selectionModel() ->selectedIndexes();
     if ( !m_plugin->modelList() )
         return ;
-    for(uint a = 0; a < m_editPatch.filesList->topLevelItemCount(); ++a) {
+    for(int a = 0; a < m_editPatch.filesList->topLevelItemCount(); ++a) {
       
         QTreeWidgetItem* item = m_editPatch.filesList->topLevelItem(a);
       
@@ -606,18 +613,28 @@ void PatchHighlighter::showToolTipForMark(QPoint pos, KTextEditor::SmartRange* m
   
   Diff2::DifferenceStringList lines;
   
+  if(m_plugin->patch()->isAlreadyApplied() && !diff->applied())
+    html += i18n("<b>Reverted.</b><br/>");
+  else if(!m_plugin->patch()->isAlreadyApplied() && diff->applied())
+    html += i18n("<b>Applied.</b><br/>");
+  
   if(diff->applied()) {
-    if(m_plugin->isReverseChange())
-      html += i18n("<b>Reverted.</b><br/>");
-    else
-      html += i18n("<b>Applied.</b><br/>");
-    
-    html += i18n("<b>Previous:</b><br/>");
-    lines = diff->sourceLines();
-  }else{
-    if(isInsertion(diff)) {
+    if(isInsertion(diff))
+    {
       html += i18n("<b>Insertion</b><br/>");
     }else{
+      if(isRemoval(diff))
+        html += i18n("<b>Removal</b><br/>");
+      html += i18n("<b>Previous:</b><br/>");
+      lines = diff->sourceLines();
+    }
+  }else{
+    if(isRemoval(diff)) {
+      html += i18n("<b>Removal</b><br/>");
+    }else{
+      if(isInsertion(diff))
+        html += i18n("<b>Insertion</b><br/>");
+      
       html += i18n("<b>Alternative:</b><br/>");
       
       lines = diff->destinationLines();
@@ -631,10 +648,10 @@ void PatchHighlighter::showToolTipForMark(QPoint pos, KTextEditor::SmartRange* m
     
     Diff2::MarkerList markers = line->markerList();
 
-    for(uint b = 0; b < markers.size(); ++b) {
-      kDebug() << "marker at " << markers[b]->offset() << ": " << markers[b]->type();
+    for(int b = 0; b < markers.size(); ++b) {
       QString spanText = Qt::escape(string.mid(currentPos, markers[b]->offset() - currentPos));
-      if(markers[b]->type() == Diff2::Marker::End) {
+      if(markers[b]->type() == Diff2::Marker::End && (currentPos != 0 || markers[b]->offset() != string.size()))
+      {
         if(a == highlightMark.first && b == highlightMark.second)
           html += "<b><span style=\"background:#FF5555\">" + spanText + "</span></b>";
         else
@@ -774,22 +791,15 @@ void PatchHighlighter::markToolTipRequested(KTextEditor::Document* , KTextEditor
 
 bool PatchHighlighter::isInsertion(Diff2::Difference* diff)
 {
-  if(!m_plugin->isReverseChange())
     return diff->sourceLineCount() == 0;
-  else
-    return diff->destinationLineCount() == 0;
 }
 
 
 bool PatchHighlighter::isRemoval(Diff2::Difference* diff)
 {
-  if(m_plugin->isReverseChange())
-    return diff->sourceLineCount() == 0;
-  else
     return diff->destinationLineCount() == 0;
 }
 
-
 void PatchHighlighter::textInserted(KTextEditor::Document* doc, KTextEditor::Range range)
 {
     if(range == doc->documentRange())
@@ -845,9 +855,15 @@ void PatchHighlighter::textInserted(KTextEditor::Document* doc, KTextEditor::Ran
         int line, lineCount;
         Diff2::DifferenceStringList lines ;
         
-        line = diff->sourceLineNumber();
-        lineCount = diff->sourceLineCount();
-        lines = diff->sourceLines();
+        if(diff->applied()) {
+          line = diff->destinationLineNumber();
+          lineCount = diff->destinationLineCount();
+          lines = diff->destinationLines();
+        } else {
+          line = diff->sourceLineNumber();
+          lineCount = diff->sourceLineCount();
+          lines = diff->sourceLines();
+        }
 
         if ( line > 0 )
             line -= 1;
@@ -876,11 +892,12 @@ void PatchHighlighter::textInserted(KTextEditor::Document* doc, KTextEditor::Ran
     }
 }
 
-PatchHighlighter::PatchHighlighter( const Diff2::DiffModel* model, IDocument* kdoc, PatchReviewPlugin* plugin ) throw( QString ) : m_doc( kdoc ), m_plugin(plugin), m_model(model) {
-//  connect( kdoc, SIGNAL( destroyed( QObject* ) ), this, SLOT( documentDestroyed() ) );
+PatchHighlighter::PatchHighlighter( const Diff2::DiffModel* model, IDocument* kdoc, PatchReviewPlugin* plugin ) throw( QString )
+  : m_doc( kdoc ), m_plugin(plugin), m_model(model)
+{
+//     connect( kdoc, SIGNAL( destroyed( QObject* ) ), this, SLOT( documentDestroyed() ) );
     connect( kdoc->textDocument(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)) );
     connect( kdoc->textDocument(), SIGNAL( destroyed( QObject* ) ), this, SLOT( documentDestroyed() ) );
-    connect( model, SIGNAL( destroyed( QObject* ) ), this, SLOT( documentDestroyed() ) );
 
     KTextEditor::Document* doc = kdoc->textDocument();
     if ( doc->lines() == 0 )
@@ -925,7 +942,9 @@ void PatchHighlighter::addLineMarker(KTextEditor::SmartRange* range, Diff2::Diff
   
     KSharedPtr<KTextEditor::Attribute> t( new KTextEditor::Attribute() );
     
-    if(!diff->applied()) {
+    bool isOriginalState = diff->applied() == m_plugin->patch()->isAlreadyApplied();
+    
+    if(isOriginalState) {
       t->setProperty( QTextFormat::BackgroundBrush, QBrush( QColor( 0, 255, 255, 20 ) ) );
     }else{
       t->setProperty( QTextFormat::BackgroundBrush, QBrush( QColor( 255, 0, 255, 20 ) ) );
@@ -936,7 +955,7 @@ void PatchHighlighter::addLineMarker(KTextEditor::SmartRange* range, Diff2::Diff
     
     KTextEditor::MarkInterface::MarkTypes mark;
     
-    if(!diff->applied()) {
+    if(isOriginalState) {
       mark = KTextEditor::MarkInterface::markType27;
       
       if(isInsertion(diff))
@@ -961,23 +980,25 @@ void PatchHighlighter::addLineMarker(KTextEditor::SmartRange* range, Diff2::Diff
     else
       lines = diff->sourceLines();
     
-    for(uint a = 0; a < lines.size(); ++a) {
+    for(int a = 0; a < lines.size(); ++a) {
       Diff2::DifferenceString* line = lines[a];
-      uint currentPos = 0;
+      int currentPos = 0;
       QString string = line->string();
       
       Diff2::MarkerList markers = line->markerList();
 
-      for(uint b = 0; b < markers.size(); ++b) { 
-//                 kDebug() << "marker at " << markers[b]->offset() << ": " << markers[b]->type();
+      for(int b = 0; b < markers.size(); ++b) { 
         if(markers[b]->type() == Diff2::Marker::End)
         {
-          KTextEditor::SmartRange * r2 = smart->newSmartRange( KTextEditor::Cursor(a + range->start().line(), currentPos), KTextEditor::Cursor(a + range->start().line(), markers[b]->offset()), range );
-          
-          KSharedPtr<KTextEditor::Attribute> t( new KTextEditor::Attribute() );
+          if(currentPos != 0 || markers[b]->offset() != string.size())
+          {
+            KTextEditor::SmartRange * r2 = smart->newSmartRange( KTextEditor::Cursor(a + range->start().line(), currentPos), KTextEditor::Cursor(a + range->start().line(), markers[b]->offset()), range );
+            
+            KSharedPtr<KTextEditor::Attribute> t( new KTextEditor::Attribute() );
 
-          t->setProperty( QTextFormat::BackgroundBrush, QBrush( QColor( 255, 0, 0, 70 ) ) );
-          r2->setAttribute( t );
+            t->setProperty( QTextFormat::BackgroundBrush, QBrush( QColor( 255, 0, 0, 70 ) ) );
+            r2->setAttribute( t );
+          }
         }
         currentPos = markers[b]->offset();
       }
@@ -1063,22 +1084,26 @@ void PatchReviewPlugin::updateKompareModel() {
     kDebug() << "updating model";
     try {
         m_modelList.reset( 0 );
+        delete m_diffSettings;
+        removeHighlighting();
+        
+        emit patchChanged();
+
+        if ( m_patch->file().isEmpty() )
+            return;
+        
         qRegisterMetaType<const Diff2::DiffModel*>( "const Diff2::DiffModel*" );
-        if ( m_diffSettings )
-            delete m_diffSettings;
         m_diffSettings = new DiffSettings( 0 );
         m_kompareInfo.reset( new Kompare::Info() );
+        m_kompareInfo->localDestination=m_patch->file().toLocalFile();
+        m_kompareInfo->localSource=m_patch->baseDir().toLocalFile();
+        
+        m_modelList.reset(new Diff2::KompareModelList( m_diffSettings.data(), new QWidget, this ));
+        m_modelList->slotKompareInfo(m_kompareInfo.get());
         
-        removeHighlighting();
-        m_modelList.reset( new Diff2::KompareModelList( m_diffSettings, *m_kompareInfo, ( QObject* ) this ) );
-        KUrl diffFile = this->diffFile();
-        if ( diffFile.isEmpty() )
-            throw "no diff file"; ;
         try {
-            ///@todo does not work with remote URLs
-            ///@todo Only reverse if the patch is currently applied
-            if ( !m_modelList->openDirAndDiff( m_patch->baseDir().toLocalFile(), diffFile.toLocalFile(), true ) )
-                throw "could not open diff " + diffFile.toLocalFile();
+            if ( !m_modelList->openDirAndDiff() )
+                throw "could not open diff " + m_patch->file().prettyUrl() + " on " + m_patch->baseDir().prettyUrl();
          } catch ( const QString & str ) {
             throw;
         } catch ( ... ) {
@@ -1086,18 +1111,25 @@ void PatchReviewPlugin::updateKompareModel() {
         }
 
         emit patchChanged();
+        
+        for(int i=0; i<m_modelList->modelCount(); i++) {
+            const Diff2::DiffModel* model=m_modelList->modelAt(i);
+            for(int j=0; j<model->differences()->count(); j++) {
+                model->differences()->at(j)->apply(m_patch->isAlreadyApplied());
+            }
+        }
 
         highlightPatch();
 
         return;
     } catch ( const QString & str ) {
-        kWarning() << "updateKompareModel:" << str;
+        KMessageBox::error(0, str, "Kompare Model Update");
     } catch ( const char * str ) {
-        kWarning() << "updateKompareModel:" << str;
+        KMessageBox::error(0, str, "Kompare Model Update");
     }
     m_modelList.reset( 0 );
-    delete m_diffSettings;
     m_kompareInfo.reset( 0 );
+    delete m_diffSettings;
     
     emit patchChanged();
 }
@@ -1201,6 +1233,8 @@ void PatchReviewPlugin::cancelReview()
   if(m_patch) {
     m_modelList.reset( 0 );
     m_patch->cancelReview();
+
+    emit patchChanged();
     
     delete m_patch;
     
@@ -1218,6 +1252,9 @@ void PatchReviewPlugin::finishReview(QList< KUrl > selection)
     if(!m_patch->finishReview(selection))
       return;
     m_modelList.reset( 0 );
+    
+    emit patchChanged();
+    
     if(!dynamic_cast<LocalPatchSource*>(m_patch))
       delete m_patch;
     
@@ -1253,7 +1290,7 @@ void PatchReviewPlugin::updateReview()
     return;
   
   //Open the diff itself
-#if HAVE_KOMPARE
+#ifdef HAVE_KOMPARE
   KUrl fakeUrl(m_patch->file());
   fakeUrl.setScheme("kdevpatch");
   IDocumentFactory* docf=ICore::self()->documentController()->factory("text/x-patch");
@@ -1266,7 +1303,7 @@ void PatchReviewPlugin::updateReview()
   ICore::self()->documentController()->openDocument(m_patch->file());
 #endif
 
-  if(m_modelList->modelCount() < 10) {
+  if(m_modelList->modelCount() < maximumFilesToOpenDirectly) {
     //Open all relates files
     for(int a = 0; a < m_modelList->modelCount(); ++a) {
       KUrl url(m_modelList->modelAt(a)->source());
@@ -1343,12 +1380,6 @@ QWidget* PatchReviewPlugin::createToolView(QWidget* parent)
     return new PatchReviewToolView(parent, this);
 }
 
-bool PatchReviewPlugin::isReverseChange() const
-{
-    return true;
-}
-
-
 #if 0
 void PatchReviewPlugin::determineState() {
     LocalPatchSourcePointer lpatch = m_patch;
