@@ -28,9 +28,9 @@ SnippetStore::SnippetStore(SnippetPlugin* plugin)
     m_self = this;
 
     const QStringList list = KGlobal::dirs()->findAllResources("data",
-        "kate/plugins/katesnippets_tng/data/*.xml", KStandardDirs::NoDuplicates)
+        "ktexteditor_snippets/data/*.xml", KStandardDirs::NoDuplicates)
                         << KGlobal::dirs()->findAllResources("data",
-        "kate/plugins/katesnippets_tng/data/ghns/*.xml", KStandardDirs::NoDuplicates);
+        "ktexteditor_snippets/ghns/*.xml", KStandardDirs::NoDuplicates);
 
     foreach(const QString& file, list ) {
         SnippetRepository* repo = new SnippetRepository(file);
@@ -96,4 +96,17 @@ bool SnippetStore::setData(const QModelIndex& index, const QVariant& value, int
     return true;
 }
 
+SnippetRepository* SnippetStore::repositoryForFile(const QString& file)
+{
+    for ( int i = 0; i < rowCount(); ++i ) {
+        if ( SnippetRepository* repo = dynamic_cast<SnippetRepository*>(item(i)) ) {
+            if ( repo->file() == file ) {
+                return repo;
+            }
+        }
+    }
+    return 0;
+}
+
+
 #include "snippetstore.moc"
