@@ -34,12 +34,46 @@
 #include <kdebug.h>
 
 #include <interfaces/iproject.h>
+#include <interfaces/iprojectcontroller.h>
+#include <interfaces/icore.h>
 #include "interfaces/iprojectfilemanager.h"
 #include <KIO/NetAccess>
+#include <language/duchain/indexedstring.h>
 
 namespace KDevelop
 {
 
+QStringList removeProjectBasePath( const QStringList& fullpath, KDevelop::ProjectBaseItem* item )
+{
+    QStringList result = fullpath;
+    if( item )
+    {
+        KDevelop::ProjectModel* model = KDevelop::ICore::self()->projectController()->projectModel();
+        QStringList basePath = model->pathFromIndex( model->indexFromItem( item ) );
+        if( basePath.count() >= fullpath.count() )
+        {
+            return QStringList();
+        }
+        for( int i = 0; i < basePath.count(); i++ )
+        {
+            result.takeFirst();
+        }
+    }
+    return result;
+}
+
+QStringList joinProjectBasePath( const QStringList& partialpath, KDevelop::ProjectBaseItem* item )
+{
+    QStringList basePath;
+    if( item )
+    {
+        KDevelop::ProjectModel* model = KDevelop::ICore::self()->projectController()->projectModel();
+        basePath = model->pathFromIndex( model->indexFromItem( item ) );
+    }
+    return basePath + partialpath;
+}
+
+
 class ProjectBaseItemPrivate
 {
 public:
@@ -73,14 +107,6 @@ class ProjectTargetItemPrivate : public ProjectBaseItemPrivate
 {
 };
 
-class WorkspaceItemPrivate
-{
-public:
-    QString name;
-    KSharedConfig::Ptr metadataConfig;
-    QString metadataDir;
-};
-
 class ProjectModelPrivate
 {
 };
@@ -111,6 +137,41 @@ KUrl ProjectBaseItem::url() const
     return KUrl();
 }
 
+KDevelop::ProjectBaseItem::ProjectItemType baseType( int type )
+{
+    if( type == KDevelop::ProjectBaseItem::Folder || type == KDevelop::ProjectBaseItem::BuildFolder )
+        return KDevelop::ProjectBaseItem::Folder;
+    if( type == KDevelop::ProjectBaseItem::Target || type == KDevelop::ProjectBaseItem::ExecutableTarget
+        || type == KDevelop::ProjectBaseItem::LibraryTarget)
+        return KDevelop::ProjectBaseItem::Target;
+
+    return static_cast<KDevelop::ProjectBaseItem::ProjectItemType>( type );
+}
+
+bool ProjectBaseItem::lessThan( const KDevelop::ProjectBaseItem* item ) const
+{
+    if(item->type() >= KDevelop::ProjectBaseItem::CustomProjectItemType ) {
+        // For custom types we want to make sure that if they override lessThan, then we
+        // prefer their lessThan implementation
+        return !item->lessThan( this );
+    }
+    KDevelop::ProjectBaseItem::ProjectItemType leftType=baseType(type()), rightType=baseType(item->type());
+    if(leftType==rightType)
+    {
+        if(leftType==KDevelop::ProjectBaseItem::File)
+        {
+            return file()->fileName().compare(item->file()->fileName(), Qt::CaseInsensitive) < 0;
+        }
+        return *this<*item;
+    }
+    else
+    {
+        return leftType<rightType;
+    }
+
+    return false;
+}
+
 IProject* ProjectBaseItem::project() const
 {
     Q_D(const ProjectBaseItem);
@@ -355,6 +416,12 @@ ProjectFileItem::ProjectFileItem( IProject* project, const KUrl & file, QStandar
     setFlags( Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable );
 }
 
+ProjectFileItem::~ProjectFileItem()
+{
+    Q_D(ProjectFileItem);
+    project()->removeFromFileSet(KDevelop::IndexedString(d->m_url));
+}
+
 void ProjectFileItem::setData(const QVariant& value, int role)
 {
     if(role==Qt::EditRole) {
@@ -386,6 +453,10 @@ const QString& ProjectFileItem::fileName() const
 void ProjectFileItem::setUrl( const KUrl& url )
 {
     Q_D(ProjectFileItem);
+    if(!d->m_url.isEmpty())
+        project()->removeFromFileSet( KDevelop::IndexedString(d->m_url) );
+    project()->addToFileSet( KDevelop::IndexedString(url) );
+    
     d->m_url = url;
     d->m_fileName = d->m_url.fileName();
     setText( d->m_fileName );
