@@ -39,12 +39,14 @@
 #include "../interfaces/icore.h"
 #include "../interfaces/idocumentcontroller.h"
 #include "../interfaces/ipartcontroller.h"
-#include "../interfaces/iuicontroller.h"
 #include "../interfaces/contextmenuextension.h"
 #include "../interfaces/context.h"
 #include "../language/interfaces/editorcontext.h"
 #include "../sublime/view.h"
+#include "../sublime/mainwindow.h"
+#include "../sublime/area.h"
 #include "core.h"
+#include "uicontroller.h"
 #include "../debugger/breakpoint/breakpointmodel.h"
 #include "../debugger/breakpoint/breakpointwidget.h"
 #include "../debugger/variable/variablewidget.h"
@@ -179,7 +181,7 @@ void DebugController::setupActions()
 {
     KActionCollection* ac = actionCollection();
 
-    KAction* action = m_continueDebugger = new KAction(KIcon("dbgrun"), i18n("&Continue"), this);
+    KAction* action = m_continueDebugger = new KAction(KIcon("media-playback-start"), i18n("&Continue"), this);
     action->setToolTip( i18n("Continues the application execution") );
     action->setWhatsThis( i18n("<b>Continue application execution</b><p>"
         "Continues the execution of your application in the "
@@ -189,7 +191,8 @@ void DebugController::setupActions()
     ac->addAction("debug_continue", action);
     connect(action, SIGNAL(triggered(bool)), this, SLOT(run()));
 
-    m_restartDebugger = action = new KAction(KIcon("dbgrestart"), i18n("&Restart"), this);
+    #if 0
+    m_restartDebugger = action = new KAction(KIcon("media-seek-backward"), i18n("&Restart"), this);
     action->setToolTip( i18n("Restart program") );
     action->setWhatsThis( i18n("<b>Restarts application</b><p>"
                                "Restarts applications from the beginning.</p>"
@@ -197,13 +200,7 @@ void DebugController::setupActions()
     action->setEnabled(false);
     connect(action, SIGNAL(triggered(bool)), this, SLOT(restartDebugger()));
     ac->addAction("debug_restart", action);
-
-
-    m_stopDebugger = action = new KAction(KIcon("process-stop"), i18n("Sto&p"), this);
-    action->setToolTip( i18n("Stop debugger") );
-    action->setWhatsThis(i18n("<b>Stop debugger</b><p>Kills the executable and exits the debugger.</p>"));
-    connect(action, SIGNAL(triggered(bool)), this, SLOT(stopDebugger()));
-    ac->addAction("debug_stop", action);
+    #endif
 
     m_interruptDebugger = action = new KAction(KIcon("media-playback-pause"), i18n("Interrupt"), this);
     action->setToolTip( i18n("Interrupt application") );
@@ -211,20 +208,20 @@ void DebugController::setupActions()
     connect(action, SIGNAL(triggered(bool)), this, SLOT(interruptDebugger()));
     ac->addAction("debug_pause", action);
 
-    m_runToCursor = action = new KAction(KIcon("dbgrunto"), i18n("Run to &Cursor"), this);
+    m_runToCursor = action = new KAction(KIcon("debug-run-cursor"), i18n("Run to &Cursor"), this);
     action->setToolTip( i18n("Run to cursor") );
     action->setWhatsThis(i18n("<b>Run to cursor</b><p>Continues execution until the cursor position is reached.</p>"));
     connect(action, SIGNAL(triggered(bool)), this, SLOT(runToCursor()));
     ac->addAction("debug_runtocursor", action);
 
 
-    m_jumpToCursor = action = new KAction(KIcon("dbgjumpto"), i18n("Set E&xecution Position to Cursor"), this);
+    m_jumpToCursor = action = new KAction(KIcon("debug-execute-to-cursor"), i18n("Set E&xecution Position to Cursor"), this);
     action->setToolTip( i18n("Jump to cursor") );
     action->setWhatsThis(i18n("<b>Set Execution Position </b><p>Set the execution pointer to the current cursor position.</p>"));
     connect(action, SIGNAL(triggered(bool)), this, SLOT(jumpToCursor()));
     ac->addAction("debug_jumptocursor", action);
 
-    m_stepOver = action = new KAction(KIcon("dbgnext"), i18n("Step &Over"), this);
+    m_stepOver = action = new KAction(KIcon("debug-step-over"), i18n("Step &Over"), this);
     action->setShortcut(Qt::Key_F10);
     action->setToolTip( i18n("Step over the next line") );
     action->setWhatsThis( i18n("<b>Step over</b><p>"
@@ -236,14 +233,14 @@ void DebugController::setupActions()
     ac->addAction("debug_stepover", action);
 
 
-    m_stepOverInstruction = action = new KAction(KIcon("dbgnextinst"), i18n("Step over Ins&truction"), this);
+    m_stepOverInstruction = action = new KAction(KIcon("debug-step-instruction"), i18n("Step over Ins&truction"), this);
     action->setToolTip( i18n("Step over instruction") );
     action->setWhatsThis(i18n("<b>Step over instruction</b><p>Steps over the next assembly instruction.</p>"));
     connect(action, SIGNAL(triggered(bool)), this, SLOT(stepIntoInstruction()));
     ac->addAction("debug_stepoverinst", action);
 
 
-    m_stepInto = action = new KAction(KIcon("dbgstep"), i18n("Step &Into"), this);
+    m_stepInto = action = new KAction(KIcon("debug-step-into"), i18n("Step &Into"), this);
     action->setShortcut(Qt::Key_F11);
     action->setToolTip( i18n("Step into the next statement") );
     action->setWhatsThis( i18n("<b>Step into</b><p>"
@@ -254,13 +251,13 @@ void DebugController::setupActions()
     ac->addAction("debug_stepinto", action);
 
 
-    m_stepIntoInstruction = action = new KAction(KIcon("dbgstepinst"), i18n("Step into I&nstruction"), this);
+    m_stepIntoInstruction = action = new KAction(KIcon("debug-step-into-instruction"), i18n("Step into I&nstruction"), this);
     action->setToolTip( i18n("Step into instruction") );
     action->setWhatsThis(i18n("<b>Step into instruction</b><p>Steps into the next assembly instruction.</p>"));
     connect(action, SIGNAL(triggered(bool)), this, SLOT(stepOverInstruction()));
     ac->addAction("debug_stepintoinst", action);
 
-    m_stepOut = action = new KAction(KIcon("dbgstepout"), i18n("Step O&ut"), this);
+    m_stepOut = action = new KAction(KIcon("debug-step-out"), i18n("Step O&ut"), this);
     action->setShortcut(Qt::Key_F12);
     action->setToolTip( i18n("Steps out of the current function") );
     action->setWhatsThis( i18n("<b>Step out</b><p>"
@@ -303,7 +300,13 @@ void DebugController::addSession(IDebugSession* session)
     
     if((Core::self()->setupFlags() & Core::NoUi)) return;
 
-    ICore::self()->uiController()->switchToArea("debug", IUiController::ThisWindow);
+
+    Sublime::MainWindow* mainWindow = Core::self()->uiControllerInternal()->activeSublimeWindow();
+    if (mainWindow->area()->objectName() != "debug") {
+        QString workingSet = mainWindow->area()->workingSet();
+        ICore::self()->uiController()->switchToArea("debug", IUiController::ThisWindow);
+        mainWindow->area()->setWorkingSet(workingSet);
+    }
 }
 
 void DebugController::clearExecutionPoint()
@@ -332,10 +335,10 @@ void DebugController::showStepInSource(const KUrl &url, int lineNum)
     kDebug() << url << lineNum;
 
     Q_ASSERT(dynamic_cast<IDebugSession*>(sender()));
-    KUrl openUrl = static_cast<IDebugSession*>(sender())->convertToLocalUrl(url);
+    QPair<KUrl,int> openUrl = static_cast<IDebugSession*>(sender())->convertToLocalUrl(qMakePair<KUrl,int>( url, lineNum ));
     KDevelop::IDocument* document = KDevelop::ICore::self()
         ->documentController()
-        ->openDocument(openUrl, KTextEditor::Cursor(lineNum, 0));
+        ->openDocument(openUrl.first, KTextEditor::Cursor(openUrl.second, 0));
 
     if( !document )
         return;
@@ -354,7 +357,7 @@ void DebugController::debuggerStateChanged(KDevelop::IDebugSession::DebuggerStat
 {
     Q_ASSERT(dynamic_cast<IDebugSession*>(sender()));
     IDebugSession* session = static_cast<IDebugSession*>(sender());
-    kDebug() << session << state;
+    kDebug() << session << state << "current" << m_currentSession;
     if (session == m_currentSession) {
         updateDebuggerState(state, session);
     }
@@ -363,6 +366,8 @@ void DebugController::debuggerStateChanged(KDevelop::IDebugSession::DebuggerStat
         if (session == m_currentSession) {
             m_currentSession = 0;
             emit currentSessionChanged(0);
+            ICore::self()->uiController()->switchToArea("code", IUiController::ThisWindow);
+            ICore::self()->uiController()->findToolView(i18n("Debug"), 0, IUiController::Raise);
         }
         session->deleteLater();
     }
@@ -379,27 +384,31 @@ void DebugController::updateDebuggerState(IDebugSession::DebuggerState state, ID
         case IDebugSession::StoppingState:
             kDebug() << "new state: stopped";
             stateChanged("stopped");
-            m_restartDebugger->setEnabled(session->restartAvaliable());
+            clearExecutionPoint();
+            //m_restartDebugger->setEnabled(session->restartAvaliable());
             break;
         case IDebugSession::StartingState:
         case IDebugSession::PausedState:
             kDebug() << "new state: paused";
             stateChanged("paused");
-            m_restartDebugger->setEnabled(session->restartAvaliable());
+            //m_restartDebugger->setEnabled(session->restartAvaliable());
             break;
         case IDebugSession::ActiveState:
             kDebug() << "new state: active";
             stateChanged("active");
-            m_restartDebugger->setEnabled(false);
+            //m_restartDebugger->setEnabled(false);
             clearExecutionPoint();
             break;
         case IDebugSession::EndedState:
             kDebug() << "new state: ended";
             stateChanged("ended");
             clearExecutionPoint();
-            m_restartDebugger->setEnabled(false);
+            //m_restartDebugger->setEnabled(false);
             break;
     }
+    if (state == IDebugSession::PausedState) {
+        ICore::self()->uiController()->activeMainWindow()->activateWindow();
+    }
 }
 
 ContextMenuExtension DebugController::contextMenuExtension( Context* context )
@@ -423,11 +432,14 @@ ContextMenuExtension DebugController::contextMenuExtension( Context* context )
     return menuExt;
 }
 
+#if 0
 void DebugController::restartDebugger() {
     if (m_currentSession) {
         m_currentSession->restartDebugger();
     }
 }
+#endif
+
 void DebugController::stopDebugger() {
     if (m_currentSession) {
         m_currentSession->stopDebugger();
