@@ -150,6 +150,31 @@ void MainWindowPrivate::activePartChanged(KParts::Part *part)
 }
 
 void MainWindowPrivate::changeActiveView(Sublime::View *view)
+{
+    //disable updates on a window to avoid toolbar flickering on xmlgui client change
+    m_mainWindow->setUpdatesEnabled( false );
+    mergeView(view);
+    m_mainWindow->setUpdatesEnabled( true );
+
+    if(!view)
+        return;
+
+    IDocument *doc = dynamic_cast<KDevelop::IDocument*>(view->document());
+    if (doc)
+    {
+        doc->activate(view, m_mainWindow);
+    }
+    else
+    {
+        //activated view is not a part document so we need to remove active part gui
+        ///@todo adymo: only this window needs to remove GUI
+//         KParts::Part *activePart = Core::self()->partController()->activePart();
+//         if (activePart)
+//             guiFactory()->removeClient(activePart);
+    }
+}
+
+void MainWindowPrivate::mergeView(Sublime::View* view)
 {
     PushPositiveValue<bool> block(m_changingActiveView, true);
 
@@ -197,21 +222,6 @@ void MainWindowPrivate::changeActiveView(Sublime::View *view)
         actionsBeforeMerge << action->objectName();;
     }
     
-    IDocument *doc = dynamic_cast<KDevelop::IDocument*>(view->document());
-    if (doc)
-    {
-        //activate part if it is not yet activated
-        doc->activate(view, m_mainWindow);
-    }
-    else
-    {
-        //activated view is not a part document so we need to remove active part gui
-        ///@todo adymo: only this window needs to remove GUI
-//         KParts::Part *activePart = Core::self()->partController()->activePart();
-//         if (activePart)
-//             guiFactory()->removeClient(activePart);
-    }
-
     // If the new view is KXMLGUIClient, add it.
     if (KXMLGUIClient* c = dynamic_cast<KXMLGUIClient*>(viewWidget))
     {
@@ -386,10 +396,12 @@ void MainWindowPrivate::setupActions()
 
 void MainWindowPrivate::setupAreaSelector()
 {
+#if 0
     if(!m_workingSetCornerWidget) {
         m_workingSetCornerWidget = Core::self()->workingSetControllerInternal()->createSetManagerWidget(m_mainWindow, true);
         m_mainWindow->setTabBarLeftCornerWidget(m_workingSetCornerWidget);
     }
+#endif
 }
 
 void MainWindowPrivate::toggleArea(bool b)
