@@ -61,7 +61,6 @@ public:
     void setupStatusBar();
     void registerStatus(QObject*);
 
-    void setupAreaSelectorActions();
     void setupAreaSelector();
     
     void tabContextMenuRequested(Sublime::View *view, KMenu* menu);
@@ -72,6 +71,7 @@ public Q_SLOTS:
     void removePlugin( KDevelop::IPlugin *plugin );
 
     void activePartChanged(KParts::Part *part);
+    void mergeView(Sublime::View *view);
     void changeActiveView(Sublime::View *view);
     void xmlguiclientDestroyed(QObject* obj);
 
@@ -102,6 +102,9 @@ public Q_SLOTS:
 
 //    void fixToolbar();
 
+    ///Returns true if we're currently changing the active view through changeActiveView()
+    bool changingActiveView() const ;
+
     bool applicationQuitRequested() const;
 
     void configureNotifications();
@@ -132,7 +135,11 @@ private:
     
     QMap<IPlugin*, KXMLGUIClient*> m_pluginCustomClients;
     
+    //LIst of all actions that were merged from the active view into the main-menu
+    QList<QPointer<QAction> > m_menuActionsFromView;
+    
     static bool s_quitRequested;
+    bool m_changingActiveView;
     /// the view of the tab that got it's context menu connected
     Sublime::View* m_tabView;
 };
