@@ -27,7 +27,7 @@ class OpenProjectDialog : public KAssistantDialog
 {
 Q_OBJECT
 public:
-    OpenProjectDialog( QWidget* parent = 0 );
+    OpenProjectDialog( const KUrl& startUrl, QWidget* parent = 0 );
     KUrl projectFileUrl();
     QString projectName();
     QString projectManager();
@@ -35,6 +35,7 @@ private slots:
     void validateOpenUrl( const KUrl& );
     void validateProjectName( const QString& );
     void validateProjectManager( const QString& );
+    void storeFileList(KIO::Job*, const KIO::UDSEntryList&);
 private:
     void validateProjectInfo();
     KUrl m_url;
@@ -42,6 +43,7 @@ private:
     QString m_projectManager;
     KPageWidgetItem* openPage;
     KPageWidgetItem* projectInfoPage;
+    QStringList m_fileList;
 };
 
 }
