@@ -59,12 +59,20 @@ Boston, MA 02110-1301, USA.
 #include "runcontroller.h"
 #include "debugcontroller.h"
 #include "documentationcontroller.h"
+#include "sourceformattercontroller.h"
+#include "projectcontroller.h"
 
 namespace KDevelop
 {
 
 static const QString pluginControllerGrp("Plugins");
 
+bool isUserSelectable( const KPluginInfo& info )
+{
+    QString loadMode = info.property( "X-KDevelop-LoadMode").toString();
+    return loadMode.isEmpty() || loadMode == "UserSelectable";
+}
+
 bool isGlobalPlugin( const KPluginInfo& info )
 {
     return info.property( "X-KDevelop-Category" ).toString() == "Global";
@@ -103,13 +111,11 @@ public:
 
     bool canUnload( const KPluginInfo& plugin )
     {
-        QStringList interfaces=plugin.property( "X-KDevelop-Interfaces" ).toStringList();
-        if(interfaces.contains("ILanguageSupport")) {
-            // Language supports can not be unloaded currently, mainly due to LanguageController and associated languages/mimetypes
-            // See: https://bugs.kde.org/show_bug.cgi?id=194337
-            return false; 
+        QStringList interfaces = plugin.property( "X-KDevelop-Interfaces" ).toStringList();
+        if( plugin.property( "X-KDevelop-LoadMode" ).toString() == "AlwaysOn" )
+        {
+            return false;
         }
-        
         foreach( const KPluginInfo& info, loadedPlugins.keys() )
         {
             if( info.pluginName() != plugin.pluginName() ) 
@@ -210,7 +216,7 @@ bool PluginController::isEnabled( const KPluginInfo& info )
     KConfigGroup grp = Core::self()->activeSession()->config()->group( pluginControllerGrp );
     bool isEnabled = grp.readEntry( info.pluginName()+"Enabled", ShellExtension::getInstance()->defaultPlugins().isEmpty() || ShellExtension::getInstance()->defaultPlugins().contains( info.pluginName() ) );
     //kDebug() << "read config:" << isEnabled << "is global plugin:" << isGlobalPlugin( info ) << "default:" << ShellExtension::getInstance()->defaultPlugins().isEmpty()  << ShellExtension::getInstance()->defaultPlugins().contains( info.pluginName() );
-    return !isGlobalPlugin( info ) || isEnabled;
+    return !isGlobalPlugin( info ) || isUserSelectable( info ) || isEnabled;
 }
 
 void PluginController::initialize()
@@ -258,14 +264,21 @@ void PluginController::initialize()
         if( isGlobalPlugin( pi ) )
         {
             QMap<QString, bool>::const_iterator it = pluginMap.constFind( pi.pluginName() );
-            if( it != pluginMap.constEnd() && it.value() )
+            if( it != pluginMap.constEnd() && ( it.value() || !isUserSelectable( pi ) ) )
             { 
                 // Plugin is mentioned in pluginmap and the value is true, so try to load it
                 loadPluginInternal( pi.pluginName() );
                 if( !grp.hasKey( pi.pluginName() + "Enabled" ) )
                 {
-                    // If plugin isn't listed yet, add it with true now
-                    grp.writeEntry( pi.pluginName()+"Enabled", true );
+                    if( isUserSelectable( pi ) )
+                    {
+                        // If plugin isn't listed yet, add it with true now
+                        grp.writeEntry( pi.pluginName()+"Enabled", true );
+                    }
+                } else if( grp.hasKey( pi.pluginName() + "Disabled" ) && !isUserSelectable( pi ) )
+                {
+                    // Remove now-obsolete entries
+                    grp.deleteEntry( pi.pluginName() + "Disabled" );
                 }
             }
         }
@@ -594,7 +607,9 @@ QList<ContextMenuExtension> PluginController::queryPluginsForContextMenuExtensio
     }
     exts << Core::self()->debugControllerInternal()->contextMenuExtension( context );
     exts << Core::self()->documentationControllerInternal()->contextMenuExtension( context );
+    exts << Core::self()->sourceFormatterControllerInternal()->contextMenuExtension( context );
     exts << Core::self()->runControllerInternal()->contextMenuExtension( context );
+    exts << Core::self()->projectControllerInternal()->contextMenuExtension( context );
     return exts;
 }
 
@@ -638,7 +653,7 @@ void PluginController::updateLoadedPlugins()
     {
         if( isGlobalPlugin( info ) )
         {
-            bool enabled = grp.readEntry( info.pluginName()+"Enabled", ( defaultPlugins.isEmpty() || defaultPlugins.contains( info.pluginName() ) ) );
+            bool enabled = grp.readEntry( info.pluginName()+"Enabled", ( defaultPlugins.isEmpty() || defaultPlugins.contains( info.pluginName() ) ) ) || !isUserSelectable( info );
             if( d->loadedPlugins.contains( info ) && !enabled ) 
             {
                 kDebug() << "unloading" << info.pluginName();
