@@ -188,6 +188,7 @@ public:
                                                                             contextItem->text() );
                 LaunchConfiguration* launch = dynamic_cast<LaunchConfiguration*>( ilaunch );
                 type->configureLaunchFromItem( launch->config(), contextItem );
+                q->setDefaultLaunch(launch);
                 //kDebug() << "created config, launching";
                 q->execute( mode->id(), launch );
             }
@@ -341,13 +342,14 @@ void RunController::initialize()
     }
 }
 
-KJob* RunController::execute(const QString& runMode, LaunchConfiguration* run)
+KJob* RunController::execute(const QString& runMode, ILaunchConfiguration* launch)
 {
-    if( !run )
+    if( !launch )
     {
         kDebug() << "execute called without launch config!";
         return 0;
     }
+    LaunchConfiguration *run = dynamic_cast<LaunchConfiguration*>(launch);
     //TODO: Port to launch framework, probably needs to be part of the launcher
     //if(!run.dependencies().isEmpty())
     //    ICore::self()->documentController()->saveAllDocuments(IDocument::Silent);
@@ -368,9 +370,9 @@ KJob* RunController::execute(const QString& runMode, LaunchConfiguration* run)
         return 0;
     }
 
-    KJob* launch = launcher->start(runMode, run);
-    registerJob(launch);
-    return launch;
+    KJob* launchJob = launcher->start(runMode, run);
+    registerJob(launchJob);
+    return launchJob;
 }
 
 void RunController::setupActions()
@@ -494,9 +496,7 @@ void KDevelop::RunController::registerJob(KJob * job)
         KAction* stopJobAction = new KAction(job->objectName().isEmpty() ? i18n("Unnamed job") : job->objectName(), this);
         stopJobAction->setData(QVariant::fromValue(static_cast<void*>(job)));
         d->stopAction->addAction(stopJobAction);
-        //TODO: This produces nasty messageboxes atm, so re-activate only after
-        //moving to a nicer job delegate
-        // connect( job, SIGNAL(finished(KJob*)), SLOT(finished(KJob*)) );
+        connect( job, SIGNAL(finished(KJob*)), SLOT(finished(KJob*)) );
         connect (stopJobAction, SIGNAL(triggered(bool)), SLOT(slotKillJob()));
 
         job->setUiDelegate( new KDialogJobUiDelegate() );
@@ -571,6 +571,7 @@ void KDevelop::RunController::finished(KJob * job)
     switch (job->error()) {
         case KJob::NoError:
         case KJob::KilledJobError:
+        case OutputJob::FailedShownError:
             break;
 
         default:
@@ -597,7 +598,15 @@ QList< KJob * > KDevelop::RunController::currentJobs() const
     return d->jobs.keys();
 }
 
-QList<LaunchConfiguration*> RunController::launchConfigurations() const
+QList<ILaunchConfiguration*> RunController::launchConfigurations() const
+{
+    QList<ILaunchConfiguration*> configs;
+    foreach (LaunchConfiguration *config, launchConfigurationsInternal())
+        configs << config;
+    return configs;
+}
+
+QList<LaunchConfiguration*> RunController::launchConfigurationsInternal() const
 {
     return d->launchConfigurations;
 }
@@ -719,6 +728,17 @@ void KDevelop::RunController::executeDefaultLaunch(const QString& runMode)
     execute( runMode, defaultLaunch() );
 }
 
+void RunController::setDefaultLaunch(LaunchConfiguration* l)
+{
+    foreach( QAction* a, d->currentTargetAction->actions() )
+    {
+        if( static_cast<LaunchConfiguration*>( qVariantValue<void*>( a->data() ) ) == l )
+        {
+            a->setChecked(true);
+            break;
+        }
+    }
+}
 
 ILaunchConfiguration* RunController::createLaunchConfiguration ( LaunchConfigurationType* type, 
                                                                  const QPair<QString,QString>& launcher, 
