@@ -34,6 +34,7 @@ class KDEVPLATFORMSHELL_EXPORT Session : public ISession
     Q_OBJECT
 public:
     static const QString cfgSessionNameEntry;
+    static const QString cfgSessionPrettyContentsEntry;
     Session( const QUuid& );
     virtual ~Session();
 
@@ -44,6 +45,8 @@ public:
 
     KUrl::List containedProjects() const;
     
+    void updateDescription();
+    
     virtual QString description() const;    
     virtual QString name() const;
     void setName( const QString& );
