@@ -56,16 +56,27 @@
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/icompletionsettings.h>
 
+#include <kdeversion.h>
+
 namespace KDevelop {
 
 struct TextDocumentPrivate {
     TextDocumentPrivate(TextDocument *textDocument)
         : encoding(""), m_textDocument(textDocument)
-        , m_loaded(false)
+        , m_loaded(false), m_addedContextMenu(0)
     {
         document = 0;
         state = IDocument::Clean;
     }
+
+    ~TextDocumentPrivate()
+    {
+        if (m_addedContextMenu) {
+            delete m_addedContextMenu;
+            m_addedContextMenu = 0;
+        }
+    }
+
     QPointer<KTextEditor::Document> document;
     IDocument::DocumentState state;
     QString encoding;
@@ -86,16 +97,22 @@ struct TextDocumentPrivate {
 
     void populateContextMenu( KTextEditor::View* v, QMenu* menu )
     {
-        ///TODO: this must not be done inside KDevelop, as it might clear actions
-        ///      from KTextEditor plugins.
-        menu->clear();
-        v->defaultContextMenu(menu);
+        if (m_addedContextMenu) {
+            foreach ( QAction* action, m_addedContextMenu->actions() ) {
+                menu->removeAction(action);
+            }
+            delete m_addedContextMenu;
+        }
 
         Context* c = new EditorContext( v, v->cursorPosition() );
         QList<ContextMenuExtension> extensions = Core::self()->pluginController()->queryPluginsForContextMenuExtensions( c );
         menu->addSeparator();
 
-        ContextMenuExtension::populateMenu(menu, extensions);
+        m_addedContextMenu = new QMenu();
+        ContextMenuExtension::populateMenu(m_addedContextMenu, extensions);
+        foreach ( QAction* action, m_addedContextMenu->actions() ) {
+            menu->addAction(action);
+        }
     }
 
     void modifiedOnDisk(KTextEditor::Document *document, bool /*isModified*/,
@@ -155,7 +172,7 @@ struct TextDocumentPrivate {
         EditorIntegrator::addDocument( m_textDocument->textDocument() );
         m_textDocument->notifyLoaded();
     }
-    
+
     void documentSaved(KTextEditor::Document* document, bool saveAs)
     {
         Q_UNUSED(document);
@@ -167,6 +184,8 @@ struct TextDocumentPrivate {
 private:
     TextDocument *m_textDocument;
     bool m_loaded;
+    // we want to remove the added stuff when the menu hides
+    QMenu* m_addedContextMenu;
 };
 
 class TextViewPrivate
@@ -270,7 +289,9 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
     view = d->document->createView(parent);
 
     if (view) {
+        #if KDE_VERSION < KDE_MAKE_VERSION(4, 4, 60)
         view->setContextMenu( view->defaultContextMenu() );
+        #endif
         connect(view, SIGNAL(contextMenuAboutToShow(KTextEditor::View*,QMenu*)), this, SLOT(populateContextMenu(KTextEditor::View*,QMenu*)));
     }
 
@@ -296,30 +317,7 @@ void TextDocument::reload()
     if (!d->document)
         return;
 
-    if (d->document->isModified())
-    {
-        if (KMessageBox::warningYesNo(Core::self()->uiController()->activeMainWindow(),
-                i18n( "The file \"%1\" is modified "
-                      "in memory. Are you sure you "
-                      "want to reload it? (Local "
-                      "changes will be lost.)", url().toLocalFile() ),
-                i18n( "Document is Modified" ) ) == KMessageBox::Yes )
-            d->document->setModified(false);
-        else
-            return ;
-    }
-
-    QList<KTextEditor::Cursor> cursors;
-    foreach (KTextEditor::View *view, d->document->views())
-        cursors << view->cursorPosition();
-
-    d->document->openUrl(url());
-
-    notifyStateChanged();
-
-    int i = 0;
-    foreach (KTextEditor::View *view, d->document->views())
-        view->setCursorPosition(cursors[i++]);
+    d->document->documentReload();
 }
 
 bool TextDocument::save(DocumentSaveMode mode)
