@@ -86,7 +86,12 @@ void WorkingSetController::initialize()
     //Load all working-sets
     KConfigGroup setConfig(Core::self()->activeSession()->config(), "Working File Sets");
     foreach(QString set, setConfig.groupList())
-        getWorkingSet(set, setConfig.group(set).readEntry<QString>("iconName", QString()));
+    {
+        if(setConfig.hasKey("iconName"))
+            getWorkingSet(set, setConfig.group(set).readEntry<QString>("iconName", QString()));
+        else
+            kDebug() << "have garbage working set with id " << set;
+    }
 }
 
 void WorkingSetController::cleanup()
@@ -595,6 +600,12 @@ void WorkingSetWidget::workingSetsChanged()
 //             kDebug() << "skipping" << set->id() << "because empty";
             continue;
         }
+
+        // Don't show working-sets that are active in an area belong to this main-window, as those
+        // can be activated directly through the icons in the tabs
+        if(!m_mini && set->hasConnectedAreas(m_mainWindow->areas()))
+             continue;
+        
 //         kDebug() << "adding button for" << set->id();
         QToolButton* butt = new WorkingSetToolButton(this, set, m_mainWindow);
         butt->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Ignored));
@@ -670,10 +681,6 @@ void WorkingSet::areaViewRemoved(Sublime::AreaIndex*, Sublime::View*) {
         return;
     }
     changed(area);
-    if (isEmpty()) {
-        kDebug() << "setting zero working set, because area" << area->objectName() << "is empty";
-        area->setWorkingSet(QString()); //Set the empty working-set if the area has been emptied
-    }
 }
 
 WorkingSet::WorkingSet(QString id, QString icon) : m_id(id), m_iconName(icon) {
