@@ -55,16 +55,33 @@ QWidget* MainWindow::customButtonForAreaSwitcher ( Area* /*area*/ )
     return 0;
 }
 
+bool MainWindow::containsView(View* view) const
+{
+    foreach(Area* area, areas())
+        if(area->views().contains(view))
+            return true;
+    return false;
+}
+
+QList< Area* > MainWindow::areas() const
+{
+    QList< Area* > areas = controller()->areas(const_cast<MainWindow*>(this));
+    if(areas.isEmpty())
+        areas = controller()->defaultAreas();
+
+    return areas;
+}
+
 void MainWindow::setupAreaSelector() {
     disconnect(d->areaSwitcher->tabBar, SIGNAL(currentChanged(int)), d, SLOT(toggleArea(int)));
     
+    d->areaSwitcher->setUpdatesEnabled(false);
+    
     d->areaSwitcher->tabBar->clearTabs();
     
     int currentIndex = -1;
     
-    QList< Area* > areas = controller()->areas(this);
-    if(areas.isEmpty())
-        areas = controller()->defaultAreas();
+    QList< Area* > areas = this->areas();
     
     QSet<QString> hadAreaName;
     
@@ -84,6 +101,11 @@ void MainWindow::setupAreaSelector() {
     }
     
     d->areaSwitcher->tabBar->setCurrentIndex(currentIndex);
+    
+    d->areaSwitcher->updateGeometry();
+    
+    d->areaSwitcher->setUpdatesEnabled(true);
+    
     connect(d->areaSwitcher->tabBar, SIGNAL(currentChanged(int)), d, SLOT(toggleArea(int)));
 }
 
@@ -126,6 +148,8 @@ void MainWindow::setArea(Area *area)
         
     connect(area, SIGNAL(viewAdded(Sublime::AreaIndex*, Sublime::View*)),
         this, SLOT(viewAdded(Sublime::AreaIndex*, Sublime::View*)));
+    connect(area, SIGNAL(viewRemoved(Sublime::AreaIndex*,Sublime::View*)),
+        this, SLOT(viewRemovedInternal(Sublime::AreaIndex*, Sublime::View*)));
     connect(area, SIGNAL(requestToolViewRaise(Sublime::View*)),
         this, SLOT(raiseToolView(Sublime::View*)));
     connect(area, SIGNAL(aboutToRemoveView(Sublime::AreaIndex*, Sublime::View*)),
