@@ -61,7 +61,6 @@
 #include "ui/revhistory/commitView.h"
 #include <language/interfaces/editorcontext.h>
 #include <vcs/vcspluginhelper.h>
-#include <memory>
 #include <KMenu>
 #include <kparts/mainwindow.h>
 
@@ -99,19 +98,19 @@ DistributedVersionControlPlugin::repositoryLocation(const KUrl &)
     return empty_cmd();
 }
 
-QList<QVariant> DistributedVersionControlPlugin::getModifiedFiles(const QString &)
+QList<QVariant> DistributedVersionControlPlugin::getModifiedFiles(const QString &, KDevelop::OutputJob::OutputJobVerbosity)
 {
     Q_ASSERT(!"Either implement DistributedVersionControlPlugin::status() or this function");
     return QList<QVariant>();
 }
 
-QList<QVariant> DistributedVersionControlPlugin::getCachedFiles(const QString &)
+QList<QVariant> DistributedVersionControlPlugin::getCachedFiles(const QString &, KDevelop::OutputJob::OutputJobVerbosity)
 {
     Q_ASSERT(!"Either implement DistributedVersionControlPlugin::status() or this function");
     return QList<QVariant>();
 }
 
-QList<QVariant> DistributedVersionControlPlugin::getOtherFiles(const QString &)
+QList<QVariant> DistributedVersionControlPlugin::getOtherFiles(const QString &, KDevelop::OutputJob::OutputJobVerbosity)
 {
     Q_ASSERT(!"Either implement DistributedVersionControlPlugin::status() or this function");
     return QList<QVariant>();
@@ -239,21 +238,15 @@ DistributedVersionControlPlugin::contextMenuExtension(Context* context)
         return ContextMenuExtension();
     }
 
-    QList<QAction*> commonActions = d->m_common->commonActions();
-
-    KMenu * menu = new KMenu(name());
-
-    if (!commonActions.empty())  {
-        menu->addActions(commonActions);
-        menu->addSeparator();
-    }
-
+    QMenu * menu = d->m_common->commonActions();
+    menu->addSeparator();
+    
     KAction *action;
-    action = new KAction(i18n("Push..."), this);
+    action = new KAction(KIcon("arrow-up-double"), i18n("Push..."), this);
     connect(action, SIGNAL(triggered()), this, SLOT(ctxPush()));
     menu->addAction(action);
 
-    action = new KAction(i18n("Pull..."), this);
+    action = new KAction(KIcon("arrow-down-double"), i18n("Pull..."), this);
     connect(action, SIGNAL(triggered()), this, SLOT(ctxPull()));
     menu->addAction(action);
 
@@ -378,7 +371,7 @@ bool DistributedVersionControlPlugin::prepareJob(DVcsJob* job, const QString& re
         return false;
     }
 
-    QFileInfo repoInfo = QFileInfo(repository);
+    QFileInfo repoInfo(repository);
     if (!repoInfo.isAbsolute()) {
         //We don't want to have empty or non-absolute pathes for working dir
         return false;
@@ -419,14 +412,14 @@ bool DistributedVersionControlPlugin::addFileList(DVcsJob* job, const KUrl::List
         //all urls should be relative to the working directory!
         //if url is relative we rely on it's relative to job->getDirectory(), so we check if it's exists
         QString file;
-        if (!url.isRelative())
+        
+        if (url.isEmpty())
+            file = '.';
+        else if (!url.isRelative())
             file = dir.relativeFilePath(url.toLocalFile());
         else
             file = url.toLocalFile();
 
-        if (file.isEmpty())
-            file = '.';
-
         args << file;
         kDebug() << "url is: " << url << "job->getDirectory(): " << workingDir << " file is: " << file;
     }
@@ -435,10 +428,10 @@ bool DistributedVersionControlPlugin::addFileList(DVcsJob* job, const KUrl::List
     return true;
 }
 
-DVcsJob* DistributedVersionControlPlugin::empty_cmd()
+DVcsJob* DistributedVersionControlPlugin::empty_cmd(KDevelop::OutputJob::OutputJobVerbosity verbosity)
 {
-    DVcsJob* j = new DVcsJob(this);
-    *j << "echo" << "-n";
+    DVcsJob* j = new DVcsJob(this, verbosity);
+    *j << "echo" << "command not implemented" << "-n";
     return j;
 }
 
