@@ -34,6 +34,7 @@
 #include "dvcsevent.h"
 #include <vcs/vcsexport.h>
 #include <vcs/vcsstatusinfo.h>
+#include <outputview/outputjob.h>
 
 class QString;
 class KDevDVCSViewFactory;
@@ -187,14 +188,14 @@ protected:
     // Additional interface to be implemented by derived plugins
     //commit dialog helpers:
     /** Returns the list of modified files (diff between workdir and index). */
-    virtual QList<QVariant> getModifiedFiles(const QString &directory);
+    virtual QList<QVariant> getModifiedFiles(const QString &directory, KDevelop::OutputJob::OutputJobVerbosity verbosity = KDevelop::OutputJob::Verbose);
     /** Returns the list of already cached files (diff between index and HEAD). */
-    virtual QList<QVariant> getCachedFiles(const QString &directory);
+    virtual QList<QVariant> getCachedFiles(const QString &directory, KDevelop::OutputJob::OutputJobVerbosity verbosity = KDevelop::OutputJob::Verbose);
     /** Files are not in the repo, but in the repository location. */
-    virtual QList<QVariant> getOtherFiles(const QString &directory);
+    virtual QList<QVariant> getOtherFiles(const QString &directory, KDevelop::OutputJob::OutputJobVerbosity verbosity = KDevelop::OutputJob::Verbose);
 
     /** empty_cmd is used when something is not implemented, but has to return any job */
-    virtual DVcsJob* empty_cmd();
+    virtual DVcsJob* empty_cmd(KDevelop::OutputJob::OutputJobVerbosity verbosity = KDevelop::OutputJob::Verbose);
 
     /** Returs the list of all commits (in all branches).
      * @see CommitView and CommitViewDelegate to see how this list is used.
