@@ -39,7 +39,7 @@ class QWidget;
 
 class VCSDiffPatchSource : public KDevelop::IPatchSource {
     public:
-    VCSDiffPatchSource(const QString& diff) ;
+    VCSDiffPatchSource(const KDevelop::VcsDiff& diff) ;
         
     virtual KUrl baseDir() const ;
     
@@ -49,6 +49,8 @@ class VCSDiffPatchSource : public KDevelop::IPatchSource {
     
     virtual void update() ;
     
+    virtual bool isAlreadyApplied() const { return true; }
+    
     KUrl m_base, m_file;
     QString m_name;
 };
@@ -56,7 +58,7 @@ class VCSDiffPatchSource : public KDevelop::IPatchSource {
 class VCSCommitDiffPatchSource : public VCSDiffPatchSource {
     Q_OBJECT
     public:
-    VCSCommitDiffPatchSource(const QString& diff, QMap<KUrl, QString> selectable, KDevelop::IBasicVersionControl* vcs);
+    VCSCommitDiffPatchSource(const KDevelop::VcsDiff& vcsdiff, QMap<KUrl, QString> selectable, KDevelop::IBasicVersionControl* vcs);
     
     ~VCSCommitDiffPatchSource() ;
     
